@@ -8,10 +8,11 @@ import numpy as np
 import pandas as pd
 import rdt
 from faker import Faker
-from xeger import Xeger
 
 from sdv.constraints.base import Constraint
+from sdv.constraints.errors import MissingConstraintColumnError
 from sdv.metadata.errors import MetadataError, MetadataNotFittedError
+from sdv.metadata.utils import strings_from_regex
 
 LOGGER = logging.getLogger(__name__)
 
@@ -456,7 +457,8 @@ class Table:
         if anonymization_mappings:
             data = data.copy()
             for name, mapping in anonymization_mappings.items():
-                data[name] = data[name].map(mapping)
+                if name in data:
+                    data[name] = data[name].map(mapping)
 
         return data
 
@@ -468,7 +470,13 @@ class Table:
                 Table to be analyzed.
         """
         LOGGER.info('Fitting table %s metadata', self.name)
-        self._field_names = self._field_names or list(data.columns)
+        if not self._field_names:
+            self._field_names = list(data.columns)
+        elif isinstance(self._field_names, set):
+            self._field_names = [field for field in data.columns if field in self._field_names]
+
+        self._dtypes = data[self._field_names].dtypes
+
         if not self._fields_metadata:
             self._fields_metadata = self._build_fields_metadata(data)
 
@@ -487,12 +495,34 @@ class Table:
         self._fit_hyper_transformer(constrained, extra_columns)
         self.fitted = True
 
-    def transform(self, data):
+    def _transform_constraints(self, data, on_missing_column='error'):
+        for constraint in self._constraints:
+            try:
+                data = constraint.transform(data)
+            except MissingConstraintColumnError:
+                if on_missing_column == 'error':
+                    raise MissingConstraintColumnError()
+
+                elif on_missing_column == 'drop':
+                    indices_to_drop = data.columns.isin(constraint.constraint_columns)
+                    columns_to_drop = data.columns.where(indices_to_drop).dropna()
+                    data = data.drop(columns_to_drop, axis=1)
+
+                else:
+                    raise ValueError('on_missing_column must be \'drop\' or \'error\'')
+
+        return data
+
+    def transform(self, data, on_missing_column='error'):
         """Transform the given data.
 
         Args:
             data (pandas.DataFrame):
                 Table data.
+            on_missing_column (str):
+                If the value is error, then a `MissingConstraintColumnError` is raised.
+                If the value is drop, then the columns involved in the constraint that
+                are present in data will be dropped.
 
         Returns:
             pandas.DataFrame:
@@ -501,37 +531,28 @@ class Table:
         if not self.fitted:
             raise MetadataNotFittedError()
 
-        fields = self.get_dtypes(ids=False)
+        fields = [field for field in self.get_dtypes(ids=False) if field in data.columns]
         LOGGER.debug('Anonymizing table %s', self.name)
         data = self._anonymize(data[fields])
 
         LOGGER.debug('Transforming constraints for table %s', self.name)
-        for constraint in self._constraints:
-            data = constraint.transform(data)
+        data = self._transform_constraints(data, on_missing_column)
 
         LOGGER.debug('Transforming table %s', self.name)
         return self._hyper_transformer.transform(data)
 
-    def _make_ids(self, name, field_metadata, length):
+    @classmethod
+    def _make_ids(cls, field_metadata, length):
         field_subtype = field_metadata.get('subtype', 'integer')
         if field_subtype == 'string':
-            regex = field_metadata.get('regex', '[a-z][A-Z]+')
-            xeger = Xeger(limit=10)
-            values = {xeger.xeger(regex) for _ in range(length)}
-            for _ in range(10):
-                done = len(values)
-                if done >= length:
-                    break
-
-                remaining = length - done
-                generate = int(round((remaining / done) * length))
-                values.update({xeger.xeger(regex) for _ in range(generate)})
-
-            else:
-                msg = 'Unable to generate {} unique values for field {} regex "{}"'.format(
-                    length, name, regex
-                )
-                raise ValueError(msg)
+            regex = field_metadata.get('regex', '[a-zA-Z]+')
+            generator, max_size = strings_from_regex(regex)
+            if max_size < length:
+                raise ValueError((
+                    'Unable to generate {} unique values for regex {}, the '
+                    'maximum number of unique values is {}.'
+                ).format(length, regex, max_size))
+            values = [next(generator) for _ in range(length)]
 
             return pd.Series(list(values)[:length])
         else:
@@ -552,22 +573,20 @@ class Table:
 
         reversed_data = self._hyper_transformer.reverse_transform(data)
 
-        for constraint in self._constraints:
+        for constraint in reversed(self._constraints):
             reversed_data = constraint.reverse_transform(reversed_data)
 
-        fields = self._fields_metadata
-        for name, dtype in self.get_dtypes(ids=True).items():
-            field_metadata = fields[name]
+        for name, field_metadata in self._fields_metadata.items():
             field_type = field_metadata['type']
             if field_type == 'id' and name not in reversed_data:
-                field_data = self._make_ids(name, field_metadata, len(reversed_data))
+                field_data = self._make_ids(field_metadata, len(reversed_data))
             elif field_metadata.get('pii', False):
                 faker = self._get_faker(field_metadata['pii_category'])
                 field_data = pd.Series([faker() for _ in range(len(reversed_data))])
             else:
                 field_data = reversed_data[name]
 
-            reversed_data[name] = field_data.dropna().astype(dtype)
+            reversed_data[name] = field_data[field_data.notnull()].astype(self._dtypes[name])
 
         return reversed_data[self._field_names]
 
@@ -587,6 +606,25 @@ class Table:
 
         return data
 
+    def make_ids_unique(self, data):
+        """Repopulate any id fields in provided data to guarantee uniqueness.
+
+        Args:
+            data (pandas.DataFrame):
+                Table data.
+
+        Returns:
+            pandas.DataFrame:
+                Table where all id fields are unique.
+        """
+        for name, field_metadata in self._fields_metadata.items():
+            if field_metadata['type'] == 'id' and not data[name].is_unique:
+                ids = self._make_ids(field_metadata, len(data))
+                ids.index = data.index.copy()
+                data[name] = ids
+
+        return data
+
     # ###################### #
     # Metadata Serialization #
     # ###################### #
@@ -623,23 +661,30 @@ class Table:
             json.dump(self.to_dict(), out_file, indent=4)
 
     @classmethod
-    def from_dict(cls, metadata_dict):
+    def from_dict(cls, metadata_dict, dtype_transformers=None):
         """Load a Table from a metadata dict.
 
         Args:
             metadata_dict (dict):
                 Dict metadata to load.
+            dtype_transformers (dict):
+                If passed, set the dtype_transformers on the new instance.
         """
-        instance = cls()
-        instance._fields_metadata = copy.deepcopy(metadata_dict['fields'])
-        instance._field_names = list(instance._fields_metadata.keys())
-        instance._constraints = copy.deepcopy(metadata_dict.get('constraints', []))
-        instance._model_kwargs = copy.deepcopy(metadata_dict.get('model_kwargs', {}))
-        instance._primary_key = metadata_dict.get('primary_key')
-        instance._sequence_index = metadata_dict.get('sequence_index')
-        instance._entity_columns = metadata_dict.get('entity_columns')
-        instance._context_columns = metadata_dict.get('context_columns')
-        instance.name = metadata_dict.get('name')
+        metadata_dict = copy.deepcopy(metadata_dict)
+        fields = metadata_dict['fields'] or {}
+        instance = cls(
+            name=metadata_dict.get('name'),
+            field_names=set(fields.keys()),
+            field_types=fields,
+            constraints=metadata_dict.get('constraints') or [],
+            model_kwargs=metadata_dict.get('model_kwargs') or {},
+            primary_key=metadata_dict.get('primary_key'),
+            sequence_index=metadata_dict.get('sequence_index'),
+            entity_columns=metadata_dict.get('entity_columns') or [],
+            context_columns=metadata_dict.get('context_columns') or [],
+            dtype_transformers=dtype_transformers,
+        )
+        instance._fields_metadata = fields
         return instance
 
     @classmethod
