@@ -2,6 +2,8 @@
 
 import logging
 import pickle
+import uuid
+from warnings import warn
 
 import numpy as np
 import pandas as pd
@@ -9,6 +11,7 @@ import pandas as pd
 from sdv.metadata import Table
 
 LOGGER = logging.getLogger(__name__)
+COND_IDX = str(uuid.uuid4())
 
 
 class NonParametricError(Exception):
@@ -233,7 +236,9 @@ class BaseTabularModel:
         """Sample a batch of rows with the given conditions.
 
         This will enter a reject-sampling loop in which rows will be sampled until
-        all of them are valid and match the requested conditions.
+        all of them are valid and match the requested conditions. If `max_retries`
+        is exceeded, it will return as many rows as it has sampled, which may be less
+        than the target number of rows.
 
         Input conditions is taken both in the raw input format, which will be used
         for filtering during the reject-sampling loop, and already transformed
@@ -277,9 +282,8 @@ class BaseTabularModel:
         counter = 0
         total_sampled = num_rows
         while num_valid < num_rows:
-            counter += 1
             if counter >= max_retries:
-                raise ValueError(f'Could not get enough valid rows within {max_retries} trials')
+                break
 
             remaining = num_rows - num_valid
             valid_probability = (num_valid + 1) / (total_sampled + 1)
@@ -289,9 +293,12 @@ class BaseTabularModel:
 
             LOGGER.info('%s valid rows remaining. Resampling %s rows', remaining, num_to_sample)
             sampled, num_valid = self._sample_rows(
-                num_to_sample, conditions, transformed_conditions, float_rtol, sampled)
+                num_to_sample, conditions, transformed_conditions, float_rtol, sampled
+            )
 
-        return sampled.head(num_rows)
+            counter += 1
+
+        return sampled.head(min(len(sampled), num_rows))
 
     def _make_conditions_df(self, conditions, num_rows):
         """Transform `conditions` into a dataframe.
@@ -329,8 +336,41 @@ class BaseTabularModel:
 
         return conditions.copy()
 
+    def _conditionally_sample_rows(self, dataframe, max_retries, max_rows_multiplier,
+                                   condition, transformed_condition, float_rtol,
+                                   graceful_reject_sampling):
+        num_rows = len(dataframe)
+        sampled_rows = self._sample_batch(
+            num_rows,
+            max_retries,
+            max_rows_multiplier,
+            condition,
+            transformed_condition,
+            float_rtol
+        )
+        num_sampled_rows = len(sampled_rows)
+
+        if num_sampled_rows < num_rows:
+            # Didn't get enough rows.
+            if len(sampled_rows) == 0:
+                error = 'No valid rows could be generated with the given conditions.'
+                raise ValueError(error)
+
+            elif not graceful_reject_sampling:
+                error = f'Could not get enough valid rows within {max_retries} trials.'
+                raise ValueError(error)
+
+            else:
+                warn(f'Only {len(sampled_rows)} rows could '
+                     f'be sampled within {max_retries} trials.')
+
+        if len(sampled_rows) > 0:
+            sampled_rows[COND_IDX] = dataframe[COND_IDX].values[:len(sampled_rows)]
+
+        return sampled_rows
+
     def sample(self, num_rows=None, max_retries=100, max_rows_multiplier=10,
-               conditions=None, float_rtol=0.01):
+               conditions=None, float_rtol=0.01, graceful_reject_sampling=False):
         """Sample rows from this table.
 
         Args:
@@ -357,6 +397,12 @@ class BaseTabularModel:
                 Maximum tolerance when considering a float match. This is the maximum
                 relative distance at which a float value will be considered a match
                 when performing reject-sampling based conditioning. Defaults to 0.01.
+            graceful_reject_sampling (bool):
+                If `False` raises a `ValueError` if not enough valid rows could be sampled
+                within `max_retries` trials. If `True` prints a warning and returns
+                as many rows as it was able to sample within `max_retries`. If no rows could
+                be generated, raises a `ValueError`.
+                Defaults to False.
 
         Returns:
             pandas.DataFrame:
@@ -374,39 +420,59 @@ class BaseTabularModel:
             if column not in self._metadata.get_fields():
                 raise ValueError(f'Invalid column name `{column}`')
 
-            if len(self._metadata.transform(conditions[[column]]).columns) == 0:
-                raise ValueError('Conditioning on column `{column}` is not possible')
-
-        transformed_conditions = self._metadata.transform(conditions)
-        condition_columns = list(transformed_conditions.columns)
-        transformed_conditions.index.name = '__condition_idx__'
+        transformed_conditions = self._metadata.transform(conditions, on_missing_column='drop')
+        condition_columns = list(conditions.columns)
+        transformed_columns = list(transformed_conditions.columns)
+        conditions.index.name = COND_IDX
+        conditions.reset_index(inplace=True)
+        transformed_conditions.index.name = COND_IDX
         transformed_conditions.reset_index(inplace=True)
-        grouped_conditions = transformed_conditions.groupby(condition_columns)
+        grouped_conditions = conditions.groupby(condition_columns)
 
         # sample
         all_sampled_rows = list()
-        for index, dataframe in grouped_conditions:
-            if not isinstance(index, tuple):
-                index = [index]
 
-            condition = conditions.loc[dataframe['__condition_idx__'].iloc[0]]
-            transformed_condition = dict(zip(condition_columns, index))
+        for group, dataframe in grouped_conditions:
+            if not isinstance(group, tuple):
+                group = [group]
 
-            sampled_rows = self._sample_batch(
-                len(dataframe),
-                max_retries,
-                max_rows_multiplier,
-                condition,
-                transformed_condition,
-                float_rtol,
-            )
-            sampled_rows['__condition_idx__'] = dataframe['__condition_idx__'].values
-            all_sampled_rows.append(sampled_rows)
+            condition_indices = dataframe[COND_IDX]
+            condition = dict(zip(condition_columns, group))
+            if transformed_conditions.empty:
+                sampled_rows = self._conditionally_sample_rows(
+                    dataframe,
+                    max_retries,
+                    max_rows_multiplier,
+                    condition,
+                    None,
+                    float_rtol,
+                    graceful_reject_sampling
+                )
+                all_sampled_rows.append(sampled_rows)
+            else:
+                transformed_conditions_in_group = transformed_conditions.loc[condition_indices]
+                transformed_groups = transformed_conditions_in_group.groupby(transformed_columns)
+                for transformed_group, transformed_dataframe in transformed_groups:
+                    if not isinstance(transformed_group, tuple):
+                        transformed_group = [transformed_group]
+
+                    transformed_condition = dict(zip(transformed_columns, transformed_group))
+                    sampled_rows = self._conditionally_sample_rows(
+                        transformed_dataframe,
+                        max_retries,
+                        max_rows_multiplier,
+                        condition,
+                        transformed_condition,
+                        float_rtol,
+                        graceful_reject_sampling
+                    )
+                    all_sampled_rows.append(sampled_rows)
 
         all_sampled_rows = pd.concat(all_sampled_rows)
-        all_sampled_rows = all_sampled_rows.set_index('__condition_idx__')
+        all_sampled_rows = all_sampled_rows.set_index(COND_IDX)
         all_sampled_rows.index.name = conditions.index.name
         all_sampled_rows = all_sampled_rows.sort_index()
+        all_sampled_rows = self._metadata.make_ids_unique(all_sampled_rows)
 
         return all_sampled_rows
 
