@@ -1,6 +1,10 @@
 """Tests for the sdv.constraints.tabular module."""
-import pandas as pd
 
+import numpy as np
+import pandas as pd
+import pytest
+
+from sdv.constraints.errors import MissingConstraintColumnError
 from sdv.constraints.tabular import (
     ColumnFormula, CustomConstraint, GreaterThan, UniqueCombinations)
 
@@ -277,6 +281,31 @@ class TestUniqueCombinations():
         })
         pd.testing.assert_frame_equal(expected_out, out)
 
+    def test_transform_not_all_columns_provided(self):
+        """Test the ``UniqueCombinations.transform`` method.
+
+        If some of the columns needed for the transform are missing, and
+        ``fit_columns_model`` is False, it will raise a ``MissingConstraintColumnError``.
+
+        Input:
+        - Table data (pandas.DataFrame)
+        Output:
+        - Raises ``MissingConstraintColumnError``.
+        """
+        # Setup
+        table_data = pd.DataFrame({
+            'a': ['a', 'b', 'c'],
+            'b': ['d', 'e', 'f'],
+            'c': ['g', 'h', 'i']
+        })
+        columns = ['b', 'c']
+        instance = UniqueCombinations(columns=columns, fit_columns_model=False)
+        instance.fit(table_data)
+
+        # Run/Assert
+        with pytest.raises(MissingConstraintColumnError):
+            instance.transform(pd.DataFrame({'a': ['a', 'b', 'c']}))
+
     def reverse_transform(self):
         """Test the ``UniqueCombinations.reverse_transform`` method.
 
@@ -316,8 +345,7 @@ class TestGreaterThan():
     def test___init___strict_false(self):
         """Test the ``GreaterThan.__init__`` method.
 
-        It is expected to create a new Constraint instance and receiving ``low`` and ``high``,
-        names of the columns that contain the low and high value.
+        The passed arguments should be stored as attributes.
 
         Input:
         - low = 'a'
@@ -338,9 +366,7 @@ class TestGreaterThan():
     def test___init___strict_true(self):
         """Test the ``GreaterThan.__init__`` method.
 
-        It is expected to create a new Constraint instance and receiving ``low`` and ``high``,
-        names of the columns that contain the low and high value. It also receives ``strict``,
-        a bool that indicates the comparison of the values should be strict.
+        The passed arguments should be stored as attributes.
 
         Input:
         - low = 'a'
@@ -359,42 +385,95 @@ class TestGreaterThan():
         assert instance._high == 'b'
         assert instance._strict is True
 
-    def test_fit(self):
+    def test_fit_int(self):
         """Test the ``GreaterThan.fit`` method.
 
-        It is expected to return the dtype of the ``high`` column.
+        The ``GreaterThan.fit`` method should only learn and store the
+        ``dtype`` of the ``high`` column as the ``_dtype`` attribute.
 
         Input:
-        - Table data (pandas.DataFrame)
-        Output:
-        - dtype of the ``high`` column.
+        - Table that contains two constrained columns with the high one
+          being made of integers.
+        Side Effect:
+        - The _dtype attribute gets `int` as the value even if the low
+          column has a different dtype.
         """
         # Setup
         instance = GreaterThan(low='a', high='b')
 
         # Run
         table_data = pd.DataFrame({
-            'a': [1, 2, 3],
+            'a': [1., 2., 3.],
             'b': [4, 5, 6],
             'c': [7, 8, 9]
         })
         instance.fit(table_data)
 
         # Asserts
-        expected = table_data['b'].dtype
-        assert instance._dtype == expected
+        assert instance._dtype.kind == 'i'
 
-    def test_is_valid_true_strict(self):
-        """Test the ``GreaterThan.is_valid`` method when the column values are valid
-        and the comparison is strict.
+    def test_fit_float(self):
+        """Test the ``GreaterThan.fit`` method.
 
-        If the columns satisfy the constraint, result is a series of ``True`` values.
+        The ``GreaterThan.fit`` method should only learn and store the
+        ``dtype`` of the ``high`` column as the ``_dtype`` attribute.
 
         Input:
-        - Table data, where the values of the ``low`` column are lower
-        than the values of the ``high`` column (pandas.DataFrame)
+        - Table that contains two constrained columns with the high one
+          being made of float values.
+        Side Effect:
+        - The _dtype attribute gets `float` as the value even if the low
+          column has a different dtype.
+        """
+        # Setup
+        instance = GreaterThan(low='a', high='b')
+
+        # Run
+        table_data = pd.DataFrame({
+            'a': [1, 2, 3],
+            'b': [4., 5., 6.],
+            'c': [7, 8, 9]
+        })
+        instance.fit(table_data)
+
+        # Asserts
+        assert instance._dtype.kind == 'f'
+
+    def test_fit_datetime(self):
+        """Test the ``GreaterThan.fit`` method.
+
+        The ``GreaterThan.fit`` method should only learn and store the
+        ``dtype`` of the ``high`` column as the ``_dtype`` attribute.
+
+        Input:
+        - Table that contains two constrained columns of datetimes.
+        Side Effect:
+        - The _dtype attribute gets `datetime` as the value.
+        """
+        # Setup
+        instance = GreaterThan(low='a', high='b')
+
+        # Run
+        table_data = pd.DataFrame({
+            'a': pd.to_datetime(['2020-01-01']),
+            'b': pd.to_datetime(['2020-01-02'])
+        })
+        instance.fit(table_data)
+
+        # Asserts
+        assert instance._dtype.kind == 'M'
+
+    def test_is_valid_strict_false(self):
+        """Test the ``GreaterThan.is_valid`` method with strict False.
+
+        If strict is False, equal values should count as valid
+
+        Input:
+        - Table with a strictly valid row, a strictly invalid row and
+          a row that has the same value for both high and low.
         Output:
-        - Series of ``True`` values (pandas.Series)
+        - False should be returned for the strictly invalid row and True
+          for the other two.
         """
         # Setup
         instance = GreaterThan(low='a', high='b', strict=True)
@@ -402,106 +481,55 @@ class TestGreaterThan():
         # Run
         table_data = pd.DataFrame({
             'a': [1, 2, 3],
-            'b': [4, 5, 6],
+            'b': [4, 2, 2],
             'c': [7, 8, 9]
         })
         out = instance.is_valid(table_data)
 
         # Assert
-        expected_out = pd.Series([True, True, True])
+        expected_out = pd.Series([True, False, False])
         pd.testing.assert_series_equal(expected_out, out)
 
-    def test_is_valid_false_strict(self):
-        """Test the ``GreaterThan.is_valid`` method when the column values are not valid
-        and the comparison is strict.
+    def test_is_valid_strict_true(self):
+        """Test the ``GreaterThan.is_valid`` method with strict True.
 
-        If the columns do not satisfy the costraint, result is a series of ``False`` values.
+        If strict is True, equal values should count as invalid.
 
         Input:
-        - Table data, where the values of the ``low`` column are higher or equal
-        than the values of the ``high`` column (pandas.DataFrame)
+        - Table with a strictly valid row, a strictly invalid row and
+          a row that has the same value for both high and low.
         Output:
-        - Series of ``False`` values (pandas.Series)
+        - True should be returned for the strictly valid row and False
+          for the other two.
         """
         # Setup
-        instance = GreaterThan(low='a', high='b', strict=True)
-
-        # Run
-        table_data = pd.DataFrame({
-            'a': [1, 2, 3],
-            'b': [1, 1, 1],
-            'c': [7, 8, 9]
-        })
-        out = instance.is_valid(table_data)
-
-        # Assert
-        expected_out = pd.Series([False, False, False])
-        pd.testing.assert_series_equal(expected_out, out)
-
-    def test_is_valid_true_not_strict(self):
-        """Test the ``GreaterThan.is_valid`` method when the column values are valid
-        and the comparison is not strict.
-
-        If the columns satisfy the constraint, result is a series of ``True`` values.
-
-        Input:
-        - Table data, where the values of the ``low`` column are lower or equal
-        than the values of the ``high`` column (pandas.DataFrame)
-        Output:
-        - Series of ``True`` values (pandas.Series)
-        """
-        # Setup
-        instance = GreaterThan(low='a', high='b')
-
-        # Run
-        table_data = pd.DataFrame({
-            'a': [1, 2, 3],
-            'b': [4, 2, 3],
-            'c': [7, 8, 9]
-        })
-        out = instance.is_valid(table_data)
-
-        # Assert
-        expected_out = pd.Series([True, True, True])
-        pd.testing.assert_series_equal(expected_out, out)
-
-    def test_is_valid_false_not_strict(self):
-        """Test the ``GreaterThan.is_valid`` method when the column values are not valid
-        and the comparison is not strict.
-
-        If the columns do not satisfy the costraint, result is a series of ``False`` values.
-
-        Input:
-        - Table data, where the values of the ``low`` column are higher
-        than the values of the ``high`` column (pandas.DataFrame)
-        Output:
-        - Series of ``True`` values (pandas.Series)
-        """
-        # Setup
-        instance = GreaterThan(low='a', high='b')
+        instance = GreaterThan(low='a', high='b', strict=False)
 
         # Run
         table_data = pd.DataFrame({
             'a': [1, 2, 3],
-            'b': [0, 1, 2],
+            'b': [4, 2, 2],
             'c': [7, 8, 9]
         })
         out = instance.is_valid(table_data)
 
         # Assert
-        expected_out = pd.Series([False, False, False])
+        expected_out = pd.Series([True, True, False])
         pd.testing.assert_series_equal(expected_out, out)
 
-    def test_transform(self):
-        """Test the ``GreaterThan.transform`` method.
+    def test_transform_int(self):
+        """Test the ``GreaterThan.transform`` method passing a high column of type int.
 
-        The ``GreaterThan.transform`` method is expected to:
-        - Transform the original table data.
+        The ``GreaterThan.transform`` method is expected to compute the distance
+        between the high and low columns and replace the high column with the
+        logarithm of the distance + 1.
 
         Input:
-        - Table data (pandas.DataFrame)
+        - Table with two columns two constrained columns at a constant distance of
+          exactly 3 and one additional dummy column.
         Output:
-        - Table data transformed (pandas.DataFrame)
+        - Same table with the high column transformed into the logarithms
+          of the distances + 1, which is np.log(4).
         """
         # Setup
         instance = GreaterThan(low='a', high='b', strict=True)
@@ -510,50 +538,204 @@ class TestGreaterThan():
         table_data = pd.DataFrame({
             'a': [1, 2, 3],
             'b': [4, 5, 6],
+            'c': [7, 8, 9],
         })
         out = instance.transform(table_data)
 
         # Assert
         expected_out = pd.DataFrame({
             'a': [1, 2, 3],
-            'b': [1.3862944, 1.3862944, 1.3862944]
+            'b': [np.log(4)] * 3,
+            'c': [7, 8, 9],
         })
         pd.testing.assert_frame_equal(out, expected_out)
 
-    def test_reverse_transform(self):
-        """Test the ``GreaterThan.reverse_transform`` method.
+    def test_transform_float(self):
+        """Test the ``GreaterThan.transform`` method passing a high column of type float.
 
-        The ``GreaterThan.reverse_transform`` method is expected to:
-        - Return the original table data.
+        The ``GreaterThan.transform`` method is expected to compute the distance
+        between the high and low columns and replace the high column with the
+        logarithm of the distance + 1.
 
         Input:
-        - Table data transformed (pandas.DataFrame)
+        - Table with two constrained columns at a constant distance of
+          exactly 3 and one additional dummy column.
         Output:
-        - Table data (pandas.DataFrame)
-        Side effects:
-        - Since ``reverse_transform`` uses the class variable ``_dtype``, the ``fit`` method
-        must be called as well.
+        - Same table with the high column transformed into the logarithms
+          of the distances + 1, which is np.log(4).
         """
         # Setup
-        table_data = pd.DataFrame({
-            'a': [1, 2, 3],
-            'b': [4, 5, 6],
-            'c': [7, 8, 9]
-        })
         instance = GreaterThan(low='a', high='b', strict=True)
-        instance.fit(table_data)
 
         # Run
-        out = instance.reverse_transform(table_data)
+        table_data = pd.DataFrame({
+            'a': [1, 2, 3],
+            'b': [4., 5., 6.],
+            'c': [7, 8, 9],
+        })
+        out = instance.transform(table_data)
 
         # Assert
         expected_out = pd.DataFrame({
             'a': [1, 2, 3],
-            'b': [55, 149, 405],
+            'b': [np.log(4)] * 3,
             'c': [7, 8, 9],
         })
         pd.testing.assert_frame_equal(out, expected_out)
 
+    def test_transform_datetime(self):
+        """Test the ``GreaterThan.transform`` method passing a high column of type datetime.
+
+        If the columns are of type datetime, ``transform`` is expected
+        to convert the timedelta distance into numeric before applying
+        the +1 and logarithm.
+
+        Input:
+        - Table with values at a distance of exactly 1 second.
+        Output:
+        - Same table with the high column transformed into the logarithms
+          of the dinstance in nanoseconds + 1, which is np.log(1_000_000_001).
+        """
+        # Setup
+        instance = GreaterThan(low='a', high='b', strict=True)
+
+        # Run
+        table_data = pd.DataFrame({
+            'a': pd.to_datetime(['2020-01-01T00:00:00', '2020-01-02T00:00:00']),
+            'b': pd.to_datetime(['2020-01-01T00:00:01', '2020-01-02T00:00:01']),
+            'c': [1, 2],
+        })
+        out = instance.transform(table_data)
+
+        # Assert
+        expected_out = pd.DataFrame({
+            'a': pd.to_datetime(['2020-01-01T00:00:00', '2020-01-02T00:00:00']),
+            'b': [np.log(1_000_000_001), np.log(1_000_000_001)],
+            'c': [1, 2],
+        })
+        pd.testing.assert_frame_equal(out, expected_out)
+
+    def test_transform_not_all_columns_provided(self):
+        """Test the ``GreaterThan.transform`` method.
+
+        If some of the columns needed for the transform are missing, it will raise
+        a ``MissingConstraintColumnError``.
+
+        Input:
+        - Table data (pandas.DataFrame)
+        Output:
+        - Raises ``MissingConstraintColumnError``.
+        """
+        # Setup
+        instance = GreaterThan(low='a', high='b', strict=True, fit_columns_model=False)
+
+        # Run/Assert
+        with pytest.raises(MissingConstraintColumnError):
+            instance.transform(pd.DataFrame({'a': ['a', 'b', 'c']}))
+
+    def test_reverse_transform_int(self):
+        """Test the ``GreaterThan.reverse_transform`` method for dtype int.
+
+        The ``GreaterThan.reverse_transform`` method is expected to:
+            - apply an exponential to the input
+            - subtract 1
+            - add the low column
+            - convert the output to integers
+
+        Input:
+        - Table with a high column that contains the constant np.log(4).
+        Output:
+        - Same table with the high column replaced by the low one + 3, as int.
+        """
+        # Setup
+        instance = GreaterThan(low='a', high='b', strict=True)
+        instance._dtype = pd.Series([1]).dtype    # exact dtype (32 or 64) depends on OS
+
+        # Run
+        transformed = pd.DataFrame({
+            'a': [1, 2, 3],
+            'b': [np.log(4)] * 3,
+            'c': [7, 8, 9]
+        })
+        out = instance.reverse_transform(transformed)
+
+        # Assert
+        expected_out = pd.DataFrame({
+            'a': [1, 2, 3],
+            'b': [4, 5, 6],
+            'c': [7, 8, 9],
+        })
+        pd.testing.assert_frame_equal(out, expected_out)
+
+    def test_reverse_transform_float(self):
+        """Test the ``GreaterThan.reverse_transform`` method for dtype float.
+
+        The ``GreaterThan.reverse_transform`` method is expected to:
+            - apply an exponential to the input
+            - subtract 1
+            - add the low column
+            - convert the output to float values
+
+        Input:
+        - Table with a high column that contains the constant np.log(4).
+        Output:
+        - Same table with the high column replaced by the low one + 3, as float values.
+        """
+        # Setup
+        instance = GreaterThan(low='a', high='b', strict=True)
+        instance._dtype = np.dtype('float')
+
+        # Run
+        transformed = pd.DataFrame({
+            'a': [1.1, 2.2, 3.3],
+            'b': [np.log(4)] * 3,
+            'c': [7, 8, 9]
+        })
+        out = instance.reverse_transform(transformed)
+
+        # Assert
+        expected_out = pd.DataFrame({
+            'a': [1.1, 2.2, 3.3],
+            'b': [4.1, 5.2, 6.3],
+            'c': [7, 8, 9],
+        })
+        pd.testing.assert_frame_equal(out, expected_out)
+
+    def test_reverse_transform_datetime(self):
+        """Test the ``GreaterThan.reverse_transform`` method for dtype datetime.
+
+        The ``GreaterThan.reverse_transform`` method is expected to:
+            - apply an exponential to the input
+            - subtract 1
+            - convert the distance to a timedelta
+            - add the low column
+            - convert the output to datetimes
+
+        Input:
+        - Table with a high column that contains the constant np.log(1_000_000_001).
+        Output:
+        - Same table with the high column replaced by the low one + one second.
+        """
+        # Setup
+        instance = GreaterThan(low='a', high='b', strict=True)
+        instance._dtype = np.dtype('<M8[ns]')
+
+        # Run
+        transformed = pd.DataFrame({
+            'a': pd.to_datetime(['2020-01-01T00:00:00', '2020-01-02T00:00:00']),
+            'b': [np.log(1_000_000_001), np.log(1_000_000_001)],
+            'c': [1, 2]
+        })
+        out = instance.reverse_transform(transformed)
+
+        # Assert
+        expected_out = pd.DataFrame({
+            'a': pd.to_datetime(['2020-01-01T00:00:00', '2020-01-02T00:00:00']),
+            'b': pd.to_datetime(['2020-01-01T00:00:01', '2020-01-02T00:00:01']),
+            'c': [1, 2]
+        })
+        pd.testing.assert_frame_equal(out, expected_out)
+
 
 def new_column(data):
     """Formula to be used for the ``TestColumnFormula`` class."""
